from collections import defaultdict
from cython.operator import dereference, postincrement



cimport cpp_read_files as cr

import sys
import numpy as np

from libc.stdint cimport uint32_t, uint16_t

from epic2.src.read_bam import read_bam

from cython.operator import dereference
from libcpp.algorithm cimport sort as stdsort
from libcpp.map cimport map as cppmap
from libcpp.algorithm cimport unique
from libcpp.vector cimport vector
from libcpp.string cimport string
from libcpp.map cimport map

cdef extern from "<algorithm>" namespace "std" nogil:
    OutputIter merge[InputIter1, InputIter2, OutputIter] (InputIter1 first1, InputIter1 last1,
                                                          InputIter2 first2, InputIter2 last2,
                                                          OutputIter result)

cimport cython

import numpy as np


@cython.boundscheck(False)
@cython.wraparound(False)
@cython.initializedcheck(False)
cpdef remove_out_of_bounds_bins(count_dict, chromsizes):

    cdef:
        uint32_t[::1] bins
        uint16_t[::1] counts
        uint32_t chromsize
        size_t i, newlength


    for chromosome, (bins, counts) in count_dict.items():
        chromsize = chromsizes[chromosome]

        # sys.stderr.write(chromosome + "\n")
        # sys.stderr.write("{} {}\n".format(bins[len(bins) - 1], chromsize))
        i = len(bins) - 1
        while (i >= 0 and bins[i] > chromsize):
            # sys.stderr.write("For {} bin {} is out of bounds ({})\n".format(chromosome, bins[i], chromsize))
            i -= 1

        if i != len(bins) - 1:
            count_dict[chromosome] = bins[:i + 1], counts[:i + 1]



@cython.boundscheck(False)
@cython.wraparound(False)
@cython.initializedcheck(False)
cpdef count_reads_per_bin(tags):

    cdef:
        uint32_t[::1] bins
        uint16_t[::1] counts
        Vector32 v
        uint32_t i
        uint32_t last = 1
        uint32_t current
        uint16_t count = 0
        uint32_t nparsed
        uint32_t last_two_equal

    bins_counts = dict()
    for k, v in tags.items():
        nparsed = 0
        count = 0

        bin_arr = np.ones(len(v), dtype=np.uint32)
        bins = bin_arr
        count_arr = np.ones(len(v), dtype=np.uint16)
        counts = count_arr

        if len(v) >= 1:
            last = v.wrapped_vector[0]

        for i in range(0, len(v)):
            current = v.wrapped_vector[i]

            if current != last:
                bins[nparsed] = last
                counts[nparsed] = count
                last = current
                count = 1
                nparsed += 1
            else:
                count += 1

        last_two_equal = v.wrapped_vector[len(v) - 2] == v.wrapped_vector[len(v) - 1]
        if last_two_equal:
            bins[nparsed] = v.wrapped_vector[len(v) - 1]
            counts[nparsed] = count
            nparsed += 1
        else:
            bins[nparsed] = v.wrapped_vector[len(v) - 1]
            counts[nparsed] = 1
            nparsed += 1

        bins_counts[k] = (bin_arr[:nparsed], count_arr[:nparsed])

    return bins_counts


@cython.boundscheck(False)
@cython.wraparound(False)
@cython.initializedcheck(False)
cdef class Vector32:

    cdef vector[uint32_t] wrapped_vector

    cdef push_back(self, uint32_t num):
        self.wrapped_vector.push_back(num)

    def sort(self):
        stdsort(self.wrapped_vector.begin(), self.wrapped_vector.end())

    def unique(self):
        self.wrapped_vector.erase(unique(self.wrapped_vector.begin(), self.wrapped_vector.end()), self.wrapped_vector.end())

    def __str__(self):
        return "[" + ", ".join([str(i) for i in self.wrapped_vector]) + "]"

    def __repr__(self):
        return str(self)

    def __len__(self):
        return self.wrapped_vector.size()

    def __iter__(self):
        # slow, only implemented to ease testing
        return (v for v in self.wrapped_vector)

    def merge(self, Vector32 other):

        cdef vector[uint32_t] o = vector[uint32_t](len(self) + len(other))
        merge(self.wrapped_vector.begin(), self.wrapped_vector.end(),
              other.wrapped_vector.begin(), other.wrapped_vector.end(),
              o.begin())

        cdef Vector32 output = Vector32()
        output.wrapped_vector = o

        return output


# @cython.boundscheck(False)
# @cython.wraparound(False)
# @cython.initializedcheck(False)
# cdef class Vector16:

#     cdef vector[uint16_t] wrapped_vector

#     cdef push_back(self, uint16_t num):
#         self.wrapped_vector.push_back(num)

#     def sort(self):
#         stdsort(self.wrapped_vector.begin(), self.wrapped_vector.end())

#     def unique(self):
#         self.wrapped_vector.erase(unique(self.wrapped_vector.begin(), self.wrapped_vector.end()), self.wrapped_vector.end())

#     def __str__(self):
#         return "[" + ", ".join([str(i) for i in self.wrapped_vector]) + "]"

#     def __repr__(self):
#         return str(self)

#     def __len__(self):
#         return self.wrapped_vector.size()

#     def __iter__(self):
#         # slow, only implemented to ease testing
#         return (v for v in self.wrapped_vector)

#     def merge(self, Vector16 other):

#         cdef vector[uint16_t] o = vector[uint16_t](len(self) + len(other))
#         merge(self.wrapped_vector.begin(), self.wrapped_vector.end(),
#               other.wrapped_vector.begin(), other.wrapped_vector.end(),
#               o.begin())

#         cdef Vector16 output = Vector16()
#         output.wrapped_vector = o

#         return output

@cython.cdivision(True)
@cython.boundscheck(False)
@cython.wraparound(False)
@cython.initializedcheck(False)
cpdef files_to_bin_counts(files, args, datatype):

    cdef:
        uint32_t bin_size = args["bin_size"]
        uint32_t half_fragment_size = args["fragment_size"] / 2
        Vector32 v
        Vector32 v2
        long[::1] bin_arr
        bytes py_bytes
        char* c_string
        cr.genome_map cpp_tags
        map[cr.key, cr.intvec].iterator it


    sum_tags = defaultdict(list)
    sys.stderr.write("Parsing {} file(s):\n".format(datatype))
    sys.stderr.flush()
    tags = dict()
    for f in files:

        sys.stderr.write("  " + f + "\n")
        sys.stderr.flush()

        py_bytes = f.encode()
        c_string = py_bytes

        if f.endswith(".bed"):
            cpp_tags = cr.read_bed(c_string)
        elif f.endswith(".bedpe"):
            cpp_tags = cr.read_bedpe(c_string)
        elif f.endswith(".bam") or f.endswith(".sam"):
            cpp_tags = read_bam(f)
        elif f.endswith(".bed.gz"):
            cpp_tags = cr.read_bed_gz(c_string)
        elif f.endswith(".bedpe.gz"):
            cpp_tags = cr.read_bedpe_gz(c_string)

        it = cpp_tags.begin();

        while (it != cpp_tags.end()):
            chromosome = dereference(it).first.first.decode()

            if chromosome not in args["chromsizes"].keys():
                postincrement(it)
                continue

            strand = chr(dereference(it).first.second)

            v = Vector32()
            v.wrapped_vector = dereference(it).second
            tags[chromosome, strand] = v

            postincrement(it)

        for v in tags.values():
            v.sort()

        if args["drop_duplicates"]:
            for k, v in tags.items():
                v.unique()

        for (chromosome, strand), v in tags.items():

            if strand == "+":
                for i in range(len(v)):
                    v.wrapped_vector[i] = v.wrapped_vector[i] + half_fragment_size
            else:
                for i in range(len(v)):
                    v.wrapped_vector[i] = v.wrapped_vector[i] - half_fragment_size

            for i in range(len(v)):
                v.wrapped_vector[i] = v.wrapped_vector[i] - (v.wrapped_vector[i] % bin_size)

            if chromosome not in sum_tags:
                sum_tags[chromosome] = v
            else:
                v2 = sum_tags[chromosome]
                sum_tags[chromosome] = v.merge(v2)

    sys.stderr.flush()

    bins_counts = count_reads_per_bin(sum_tags)

    remove_out_of_bounds_bins(bins_counts, args["chromsizes"])

    return bins_counts



cpdef add_reads_to_dict(f, chromosomes):

    genome = dict()
    cdef Vector32 v

    for line in open(f):
        chromosome, left, right, _, _, strand = line.split()

        if chromosome not in chromosomes:
            continue

        if strand == "+":
            five_end = <uint32_t> int(left)
        else:
            five_end = <uint32_t> int(right)

        if (chromosome, strand) in genome:
            v = genome[chromosome, strand]
            v.wrapped_vector.push_back(five_end)
        else:
            v = Vector32()
            v.wrapped_vector.push_back(five_end)
            genome[chromosome, strand] = v

    return genome



######## this was actually slower!

# cpdef add_reads_to_dict(f):

#     genome = dict()
#     cdef:
#         Vector v
#         FILE *f_handle
#         char chromosome [10]
#         char strand [2]
#         uint32_t left
#         uint32_t right

#     fp = fopen(f.encode(), "r")

#     while (
#             fscanf(fp, "%s\t%d\t%d\t%*s\t%*d\t%s\n", chromosome, &left, &right, strand) != EOF
#     ):


#         # pruint32_t("----")
#         # pruint32_t("chromosome is", chromosome)
#         # pruint32_t("strand is", strand)
#         # pruint32_t("left is", left)
#         # pruint32_t("right is", right)
#         if strand == b"+":
#             five_end = left
#         else:
#             five_end = right

#         # pruint32_t("five end is ", five_end)

#         if (chromosome, strand) in genome:
#             v = genome[chromosome, strand]
#             v.wrapped_vector.push_back(five_end)
#         else:
#             v = Vector()
#             v.wrapped_vector.push_back(five_end)
#             genome[chromosome, strand] = v

#     return genome




from cython.operator import dereference, postincrement
from libc.stdint cimport uint32_t, uint16_t, int32_t, int64_t

from cython.operator import dereference
from libcpp.algorithm cimport sort as stdsort
from libcpp.map cimport map as cppmap
from libcpp.algorithm cimport unique
from libcpp.vector cimport vector
from libcpp.string cimport string
from libcpp.pair cimport pair
from libcpp.map cimport map

cdef extern from "<utility>" namespace "std" nogil:
    pair[T,U] make_pair[T,U](T&,U&)

ctypedef vector[uint32_t] intvec
ctypedef pair [string, char] key
ctypedef pair [int, char] intkey
ctypedef map[intkey, intvec] genome_map_int
ctypedef map[key, intvec] genome_map


# include "<htslib>"


# cpdef read_bam(filename):

#     cdef:


import pysam


from pysam.libcalignmentfile cimport AlignmentFile, AlignedSegment


# ctypedef struct Alignment:
#     int32_t start
#     uint32_t alen
#     uint32_t reference_id
#     int64_t flag

cpdef read_bam(filename):

    cdef:
        uint32_t flag
        int32_t start
        int32_t end
        int32_t length
        uint32_t is_strand
        char forward = "+"
        char reverse = "-"
        char strand
        string chromosome
        int chromosome_id
        intkey chrom_strand
        key chrom_strand_fixed
        uint32_t five_end
        genome_map_int genome
        genome_map genome_fixed
        AlignmentFile samfile
        AlignedSegment a

    samfile = pysam.AlignmentFile(filename, "rb")


    for a in samfile:
        flag = a.flag

        # https://broadinstitute.github.io/picard/explain-flags.html
        if flag & 0x4 or flag & 0x200 or flag & 0x400:
            continue

        is_reverse = flag & 0x10

        start = a.reference_start

        end = start + a.alen

        if start < 0 or end < 0:
            continue

        chromosome_id = a.reference_id
        if is_reverse:
            chrom_strand = make_pair(<int>chromosome_id, <char>reverse)
            genome[chrom_strand].push_back(end + 1)
        else:
            chrom_strand = make_pair(<int>chromosome_id, <char>forward)
            genome[chrom_strand].push_back(start + 1)


    it = genome.begin();

    while (it != genome.end()):
        chromosome_id = dereference(it).first.first
        chromosome = samfile.get_reference_name(chromosome_id).encode("utf-8")

        strand = dereference(it).first.second

        if chr(strand) == "+":
            chrom_strand_fixed = make_pair(<string>chromosome, <char>forward)
            genome_fixed[chrom_strand_fixed] = dereference(it).second
        else:
            chrom_strand_fixed = make_pair(<string>chromosome, <char>reverse)
            genome_fixed[chrom_strand_fixed] = dereference(it).second

        postincrement(it)

    return genome_fixed



from scipy.stats import poisson, rankdata
import cython
from libcpp cimport bool

from libcpp.algorithm cimport sort as stdsort
from libc.math cimport log2
import numpy as np

from libc.stdint cimport uint32_t, uint16_t
from epic2.src.statistics import compute_window_score
from libcpp.vector cimport vector
# from libc.math cimport min as cmin

from natsort import natsorted


cdef struct island:
    uint32_t start
    uint32_t end
    uint16_t chip_count
    uint16_t input_count
    float score
    float p_value
    float fold_change


# cdef bool compare_p_value(const island &lhs, const island &rhs):

#     return lhs.p_value < rhs.p_value



cdef class IslandVector:

    cdef vector[island] wrapped_vector

    cdef push_back(self, island i):
        self.wrapped_vector.push_back(i)

    def __str__(self):
        return "[" + ", ".join([str(i) for i in self.wrapped_vector]) + "]"

    def __repr__(self):
        return str(self)

    def __len__(self):
        return self.wrapped_vector.size()

    def __iter__(self):
        # slow, only implemented to ease testing
        return (v for v in self.wrapped_vector)

    # def p_value_sort(self):
    #     stdsort(self.wrapped_vector.begin(), self.wrapped_vector.end(), &compare_p_value)



@cython.boundscheck(False)
@cython.wraparound(False)
@cython.initializedcheck(False)
def find_islands(bins_counts, int gaps_allowed, int bin_size, float score_threshold, uint32_t island_enriched_threshold, float average_window_readcount):

    _poisson = poisson(average_window_readcount)

    cdef:
        pass
        float slightly_less = score_threshold - 0.0000000001
        float score
        int i
        int j
        uint16_t count
        uint32_t _bin
        uint32_t[::1] bins = np.ones(1, dtype=np.uint32)
        uint16_t[::1] counts = np.ones(1, dtype=np.uint16)
        uint32_t dist
        island _island
        island current_island
        IslandVector v
        uint32_t distance_allowed = (gaps_allowed * bin_size) + 2

    chromosomes = set(bins_counts.keys())

    island_dict = dict()

    for chromosome in chromosomes:
        v = IslandVector()
        i = 0

        # TODO: if chromo not in chromsizes, remove
        bins, counts = bins_counts[chromosome]

        count = counts[0]
        _bin = bins[0]
        score = compute_window_score(count, _poisson)

        current_island = [_bin, _bin + bin_size - 1, count, 0, score, 0, 0]
        for i in range(1, len(bins)):

            count = counts[i]

            _bin = bins[i]

            dist = _bin - current_island.end

            if dist <= distance_allowed:
                current_island.end = bins[i] + bin_size - 1
                current_island.chip_count += counts[i]
                current_island.score += compute_window_score(counts[i], _poisson)
            else:
                if current_island.score > slightly_less and current_island.chip_count > island_enriched_threshold:
                    v.push_back(current_island)

                score = compute_window_score(counts[i], _poisson)
                current_island = [_bin, bins[i] + bin_size - 1, count, 0, score, 0, 0]

        # takes care of last or potential single bin
        if current_island.score > slightly_less and current_island.chip_count > island_enriched_threshold:
            v.push_back(current_island)

        island_dict[chromosome] = v

        del bins_counts[chromosome]

    return island_dict

@cython.boundscheck(False)
@cython.wraparound(False)
@cython.initializedcheck(False)
@cython.cdivision(True)
def compute_fdr(islands, b_bins_counts, int chip_library_size, int control_library_size, float effective_genome_fraction, float fdr_cutoff):

    cdef:
        int i
        int j
        int number_bins
        # island *_island
        island _island
        IslandVector _islands
        IslandVector all_islands = IslandVector()
        uint16_t count
        uint32_t _bin
        uint32_t[::1] bins = np.ones(1, dtype=np.uint32)
        uint16_t[::1] counts = np.ones(1, dtype=np.uint16)
        float scaling_factor = float(chip_library_size) / float(control_library_size)
        float zero_scaler = (control_library_size / effective_genome_fraction)
        float fdr
        float average
        int num_islands
        int counter

    # print("chip_library_size", chip_library_size)
    # print("control_library_size", control_library_size)
    # print("float(chip_library_size) / float(control_library_size)", float(chip_library_size) / float(control_library_size))
    sf = poisson.sf
    chromosomes = natsorted(set(islands.keys()))
    num_islands_per_chrom = [len(v) for _, v in natsorted(islands.items())]
    # print("scaling_factor", scaling_factor)
    # print("zero_scaler", zero_scaler)
    for chromosome in chromosomes:
        j = 0
        _islands = islands[chromosome]
        # print("chr", chromosome, len(_islands), _islands)

        if chromosome not in b_bins_counts:
            continue

        bins, counts = b_bins_counts[chromosome]
        number_bins = len(bins)

        for i in range(len(_islands)):
            # _island = &_islands.wrapped_vector[i]
            _island = _islands.wrapped_vector[i]

            # not overlapping
            while (bins[j] < _island.start and j < number_bins):
                # print("bins[j]", bins[j], "_island.start", _island.start)
                j += 1

            # overlapping
            while (bins[j] < _island.end and bins[j] >= _island.start and j < number_bins):
                _island.input_count += counts[j]
                j += 1

            ##### COMPUTE P ########
            if _island.input_count > 0:
                average = _island.input_count * scaling_factor
            else:
                average = (_island.end - _island.start + 1) * zero_scaler
                average = min(0.25, average) * scaling_factor

            _island.fold_change = log2(_island.chip_count / average)

            if _island.chip_count > average:
                _island.p_value = sf(_island.chip_count, average)
            else:
                _island.p_value = 1

            all_islands.push_back(_island)

        del b_bins_counts[chromosome]


    ranks = rankdata(np.array([_island.p_value for _island in all_islands], dtype=np.int))

    counter = 0
    num_islands = len(all_islands)

    print("\t".join(["Chromosome", "Start", "End", "PValue", "Score", "Strand", "ChIPCount", "InputCount", "FDR", "log2(FoldChange)"]))
    for chromosome, chromosome_size in zip(chromosomes, num_islands_per_chrom):

        i = 0
        for i in range(chromosome_size):

            #### compute fdr #####
            _island = all_islands.wrapped_vector[i + counter]
            fdr = _island.p_value * num_islands / ranks[i + counter]
            if fdr > 1:
                fdr = 1

            if fdr <= fdr_cutoff:
                print("\t".join(str(e) for e in [chromosome, _island.start, _island.end, _island.p_value,
                                                 min(1000, _island.fold_change * 100), ".", _island.chip_count, _island.input_count, fdr, _island.fold_change]))

        counter += chromosome_size


def write_islands(islands, float average_window_readcount, float fdr_cutoff):

    cdef:
        island _island
        IslandVector _islands
        IslandVector all_islands = IslandVector()
        int i
        int num_islands
        int counter


    chromosomes = natsorted(set(islands.keys()))
    num_islands_per_chrom = [len(v) for _, v in natsorted(islands.items())]
    _poisson = poisson(average_window_readcount)
    print("\t".join(["Chromosome", "Start", "End", "PValue", "ChIPCount", "Strand", "FDR"]))
    for chromosome in chromosomes:
        _islands = islands[chromosome]
        for _island in _islands:
            _island.p_value = _poisson.pmf(_island.chip_count)
            all_islands.wrapped_vector.push_back(_island)

    ranks = rankdata(np.array([_island.p_value for _island in all_islands], dtype=np.int))
    counter = 0
    num_islands = len(all_islands)
    for chromosome, chromosome_size in zip(chromosomes, num_islands_per_chrom):

        i = 0
        for i in range(chromosome_size):

            #### compute fdr #####
            _island = all_islands.wrapped_vector[i + counter]

            fdr = _island.p_value * num_islands / ranks[i + counter]
            if fdr > 1:
                fdr = 1
            if fdr <= fdr_cutoff:
                print("\t".join(str(e) for e in [chromosome, _island.start, _island.end, _island.p_value, _island.chip_count, ".", fdr]))



from os.path import basename
from natsort import natsorted
from collections import OrderedDict

from libc.stdint cimport uint32_t, uint16_t
import numpy as np
import sys
from re import search, IGNORECASE
import pkg_resources


def get_closest_readlength(estimated_readlength):
    # type: (int) -> int
    """Find the predefined readlength closest to the estimated readlength.

    In the case of a tie, choose the shortest readlength."""

    readlengths = [36, 50, 75, 100]
    differences = [abs(r - estimated_readlength) for r in readlengths]
    min_difference = min(differences)
    index_of_min_difference = [i
                               for i, d in enumerate(differences)
                               if d == min_difference][0]

    return readlengths[index_of_min_difference]


def find_readlength(args):
    # type: (Namespace) -> int
    """Estimate length of reads based on 1000 first."""

    # from subprocess import check_output
    # import pandas as pd
    # from io import BytesIO
    _file = args["treatment"][0]


    arr = np.zeros(100, dtype=np.uint32)

    cdef:
        int i = 0
        uint32_t start
        uint32_t end

    if _file.endswith(".bed"):
        for line in open(_file):
            _start, _end = line.split()[1:3]
            arr[i] = int(_end) - int(_start)
            i += 1
            if i == 100:
                break

    elif _file.endswith(".bam") or _file.endswith(".sam"):

        import pysam

        if _file.endswith(".bam"):
            samfile = pysam.AlignmentFile(_file, "rb")
        elif _file.endswith(".sam"):
            samfile = pysam.AlignmentFile(_file, "r")

        for a in samfile:
            if a.alen == None: continue

            arr[i] = a.alen
            i += 1
            if i == 100:
                break

        print(arr)


    return get_closest_readlength(np.median(arr))


    # filereader = "cat "
    # if bed_file.endswith(".gz"):
    #     filereader = "gzcat "
    # elif bed_file.endswith(".bz2"):
    #     filereader = "bzgrep "
    # elif bed_file.endswith(".bam"):
    #     filereader = "samtools view "

    # command = filereader + "{} | head -10000".format(bed_file)
    # output = check_output(command, shell=True)

    # df = pd.read_table(
    #     BytesIO(output),
    #     header=None,
    #     usecols=[1, 2],
    #     sep="\t",
    #     names=["Start", "End"])

    # readlengths = df.End - df.Start
    # mean_readlength = readlengths.mean()
    # median_readlength = readlengths.median()
    # max_readlength = readlengths.max()
    # min_readlength = readlengths.min()

    # logging.info((
    #     "Used first 10000 reads of {} to estimate a median read length of {}\n"
    #     "Mean readlength: {}, max readlength: {}, min readlength: {}.").format(
    #         bed_file, median_readlength, mean_readlength, max_readlength,
    #         min_readlength))




def get_genome_size_file(genome):

    genome_names = pkg_resources.resource_listdir("epic", "scripts/chromsizes")
    name_dict = {n.lower().replace(".chromsizes", ""): n for n in genome_names}

    # # No try/except here, because get_egs would already have failed if genome
    # # did not exist
    genome_exact = name_dict[genome.lower()]

    return pkg_resources.resource_filename(
        "epic2", "chromsizes/{}".format(genome_exact))


def create_genome_size_dict(genome):
    # type: (str) -> Dict[str,int]
    """Creates genome size dict from string containing data."""

    size_file = get_genome_size_file(genome)
    size_lines = open(size_file).readlines()

    size_dict = {}
    for line in size_lines:
        genome, length = line.split()
        size_dict[genome] = int(length)

    return size_dict


def create_genome_size_dict_custom_genome(chromsizes):

    chromosome_lengths = [l.split() for l in open(chromsizes).readlines()]

    od = OrderedDict()          # type: OrderedDict[str, int]

    for c, l in natsorted(chromosome_lengths):
        od[c] = int(l)

    return od


def get_effective_genome_fraction(genome, read_length):

    genome_names = pkg_resources.resource_listdir("epic2",
                                                  "effective_sizes")

    name_dict = {n.lower(): n for n in genome_names}

    # # No try/except here, because get_egs would already have failed if genome
    # # did not exist
    genome_exact = name_dict[genome.lower() + "_" + str(read_length) + ".txt"]


    try:
        egf = pkg_resources.resource_string( # type: ignore
            "epic2", "effective_sizes/{}".format(
                genome_exact)).split()[-1].decode()
    except KeyError:

        genome_list = "\n".join([basename(g) for g in genome_names])

        raise Exception("Genome " + genome +
            " not found.\n These are the available genomes: " + genome_list +
            "\nIf yours is not there, please request it at github.com/endrebak/epic2 .")

    return float(egf)


def egl_and_chromsizes(args):

    have_chromsizes = args["chromsizes"] != None
    have_effective_genome_fraction = args["effective_genome_fraction"] != None

    if have_chromsizes or have_effective_genome_fraction:
        assert have_chromsizes
        assert have_effective_genome_fraction

        chromsizes = create_genome_size_dict_custom_genome(args["chromsizes"])
        egf = args["effective_genome_fraction"]
        egl = egf * sum(chromsizes.values())
        genome_length = sum(chromsizes.values())
    else:
        read_length = find_readlength(args)
        chromsizes = create_genome_size_dict(args["genome"])
        egf = get_effective_genome_fraction(args["genome"], read_length)
        genome_length = sum(chromsizes.values())
        egl = egf * genome_length

    sys.stderr.write("Using an effective genome length of ~{} * 1e6 \n\n".format(int(egl/1e6)))

    return egl, chromsizes
